"""Pytest configuration and fixtures."""

import json
from pathlib import Path
from typing import List, Dict, Any

import pytest


FIXTURES_DIR = Path(__file__).parent / "fixtures"


@pytest.fixture
def system_messages() -> List[str]:
    """Load system message samples for filter testing."""
    path = FIXTURES_DIR / "system_messages.json"
    with open(path, "r", encoding="utf-8") as f:
        return json.load(f)


@pytest.fixture
def real_prompts() -> List[str]:
    """Load real user prompt samples."""
    path = FIXTURES_DIR / "real_prompts.json"
    with open(path, "r", encoding="utf-8") as f:
        return json.load(f)


@pytest.fixture
def high_quality_prompts() -> List[Dict[str, Any]]:
    """Load high quality prompt samples with metadata."""
    path = FIXTURES_DIR / "high_quality_prompts.json"
    with open(path, "r", encoding="utf-8") as f:
        return json.load(f)


@pytest.fixture
def low_quality_prompts() -> List[Dict[str, Any]]:
    """Load low quality prompt samples with metadata."""
    path = FIXTURES_DIR / "low_quality_prompts.json"
    with open(path, "r", encoding="utf-8") as f:
        return json.load(f)


def load_fixture(name: str) -> Any:
    """Helper function to load fixtures by name."""
    path = FIXTURES_DIR / f"{name}.json"
    with open(path, "r", encoding="utf-8") as f:
        return json.load(f)



"""Tests for the new scoring model."""

import pytest
from claude_x.scoring import (
    calculate_structure_score,
    calculate_context_score,
    calculate_efficiency_score,
    calculate_diversity_score,
    calculate_productivity_score,
    calculate_composite_score_v2,
    detect_console_log_ratio,
    calculate_context_dependency_penalty,
)


class TestConsoleLogDetection:
    """Tests for detect_console_log_ratio function."""

    def test_pure_console_log(self):
        """Should detect console log output."""
        prompt = """[Profile]  Page loaded: {profileId: '4z4z-newwwww', membershipContent: false}
client-logger.ts:24 [Profile]  📊 State check {hasInitialized: true}
logger-console.ts:19 [GET] /v2/article/@4z4z-newwwww"""
        log_ratio, has_question = detect_console_log_ratio(prompt)
        assert log_ratio > 0.5, f"Expected log_ratio > 0.5, got {log_ratio}"
        assert not has_question, "Should not detect question"

    def test_console_log_with_question(self):
        """Should detect console log with question."""
        prompt = """[Profile]  Page loaded: {profileId: '4z4z-newwwww'}
client-logger.ts:24 [Profile]  📊 State check {hasInitialized: true}
이게 왜 이렇게 되는 거야? 에러 원인이 뭘까?"""
        log_ratio, has_question = detect_console_log_ratio(prompt)
        assert log_ratio > 0.3, f"Expected log_ratio > 0.3, got {log_ratio}"
        assert has_question, "Should detect question"

    def test_normal_prompt(self):
        """Should not detect console log in normal prompts."""
        prompt = "LoginForm.tsx 컴포넌트에 validation 추가해줘"
        log_ratio, has_question = detect_console_log_ratio(prompt)
        assert log_ratio < 0.3, f"Expected log_ratio < 0.3, got {log_ratio}"

    def test_empty_prompt(self):
        """Should handle empty prompt."""
        log_ratio, has_question = detect_console_log_ratio("")
        assert log_ratio == 0.0
        assert not has_question


class TestContextDependencyPenalty:
    """Tests for calculate_context_dependency_penalty function."""

    def test_context_dependent_prompt(self):
        """Should penalize context-dependent prompts."""
        prompts = [
            "위의 방법중 어떤게 좋아?",
            "아까 말한 그거 해줘",
            "그건 뭐야?",
            "ㅇㅇ 그거",
        ]
        for prompt in prompts:
            penalty = calculate_context_dependency_penalty(prompt)
            assert penalty > 0, f"Prompt '{prompt}' should have penalty"

    def test_independent_prompt(self):
        """Should not penalize independent prompts."""
        prompt = "LoginForm.tsx 컴포넌트에 validation을 추가해줘"
        penalty = calculate_context_dependency_penalty(prompt)
        assert penalty == 0.0, f"Expected 0.0 penalty, got {penalty}"

    def test_very_short_context_dependent(self):
        """Should heavily penalize very short context-dependent prompts."""
        prompt = "그거 해줘"
        penalty = calculate_context_dependency_penalty(prompt)
        assert penalty >= 2.0, f"Expected >= 2.0 penalty, got {penalty}"

    def test_empty_prompt(self):
        """Should handle empty prompt."""
        assert calculate_context_dependency_penalty("") == 0.0


class TestStructureScore:
    """Tests for calculate_structure_score function."""

    def test_good_structure_prompt(self):
        """Should give high score to well-structured prompts."""
        prompt = "LoginForm.tsx 컴포넌트에 비밀번호 유효성 검사 추가해줘"
        score = calculate_structure_score(prompt)
        assert score >= 4.0, f"Expected >= 4.0, got {score}"

    def test_great_structure_prompt(self):
        """Should give very high score to excellent prompts."""
        prompt = (
            "현재 src/components/LoginForm.tsx에서 비밀번호 유효성 검사를 "
            "추가해줘. 최소 8자 이상, 특수문자 포함 조건으로 해줘. "
            "기존 이메일 검증 방식처럼 만들어줘."
        )
        score = calculate_structure_score(prompt)
        assert score >= 6.0, f"Expected >= 6.0, got {score}"

    def test_poor_structure_prompt(self):
        """Should give low score to vague prompts."""
        prompt = "fix it"
        score = calculate_structure_score(prompt)
        assert score <= 3.0, f"Expected <= 3.0, got {score}"

    def test_very_short_prompt(self):
        """Should penalize very short prompts."""
        prompt = "ok"
        score = calculate_structure_score(prompt)
        assert score <= 2.0, f"Expected <= 2.0, got {score}"

    def test_empty_prompt(self):
        """Should return 0 for empty prompt."""
        assert calculate_structure_score("") == 0.0
        assert calculate_structure_score(None) == 0.0

    def test_goal_patterns_korean(self):
        """Should detect Korean goal patterns."""
        prompts = ["버그를 수정해줘", "기능을 추가해줘", "코드를 리팩토링해줘"]
        for prompt in prompts:
            score = calculate_structure_score(prompt)
            assert score >= 2.0, f"Prompt '{prompt}' should have goal detected"

    def test_goal_patterns_english(self):
        """Should detect English goal patterns."""
        prompts = ["please fix the bug", "create a new component", "implement authentication"]
        for prompt in prompts:
            score = calculate_structure_score(prompt)
            assert score >= 2.0, f"Prompt '{prompt}' should have goal detected"

    def test_console_log_only_prompt(self):
        """Should give low score for console-log-only prompts."""
        prompt = """[Profile]  Page loaded: {profileId: '4z4z-newwwww', membershipContent: false}
client-logger.ts:24 [Profile]  📊 State check {hasInitialized: true}
logger-console.ts:19 [GET] /v2/article/@4z4z-newwwww"""
        score = calculate_structure_score(prompt)
        assert score <= 2.0, f"Console log only should have low score, got {score}"

    def test_console_log_with_question(self):
        """Should give moderate score for logs with question (not a great template)."""
        prompt = """[Profile]  Page loaded: {profileId: '4z4z-newwwww'}
이 로그 보면 에러가 발생하는데 원인이 뭘까?"""
        score = calculate_structure_score(prompt)
        # Logs with question are better than logs alone, but still not great templates
        assert 2.0 <= score <= 4.0, f"Logs with question should have moderate score, got {score}"

    def test_context_dependent_prompt_penalized(self):
        """Should penalize context-dependent prompts."""
        prompts = [
            "위의 방법중 어떤게 좋아?",
            "아까 말한 그거 해줘",
            "그건 뭐야?",
        ]
        for prompt in prompts:
            score = calculate_structure_score(prompt)
            assert score <= 4.0, f"Context-dependent '{prompt}' should have low score, got {score}"


class TestContextScore:
    """Tests for calculate_context_score function."""

    def test_context_rich_prompt(self):
        """Should give high score to context-rich prompts."""
        prompt = (
            "현재 React 프로젝트의 src/components/Login.tsx에서 "
            "TypeError: Cannot read property 'name' of undefined 에러가 발생해"
        )
        score = calculate_context_score(prompt)
        assert score >= 6.0, f"Expected >= 6.0, got {score}"

    def test_context_poor_prompt(self):
        """Should give low score to context-poor prompts."""
        prompt = "수정해줘"
        score = calculate_context_score(prompt)
        assert score <= 2.0, f"Expected <= 2.0, got {score}"

    def test_empty_prompt(self):
        """Should return 0 for empty prompt."""
        assert calculate_context_score("") == 0.0

    def test_file_path_detection(self):
        """Should detect file paths."""
        prompts = [
            "src/components/Button.tsx",
            "components/Header.jsx",
            "utils/helper.py",
        ]
        for prompt in prompts:
            score = calculate_context_score(prompt)
            assert score >= 2.0, f"File path in '{prompt}' should be detected"

    def test_technology_detection(self):
        """Should detect technology mentions."""
        prompts = [
            "React 컴포넌트를 만들어줘",
            "TypeScript 타입을 추가해줘",
            "GraphQL 쿼리를 작성해줘",
        ]
        for prompt in prompts:
            score = calculate_context_score(prompt)
            assert score >= 2.0, f"Tech in '{prompt}' should be detected"

    def test_code_block_detection(self):
        """Should detect code blocks."""
        prompt = "```typescript\nconst x = 1;\n```"
        score = calculate_context_score(prompt)
        assert score >= 2.0

    def test_inline_code_detection(self):
        """Should detect inline code."""
        prompt = "`useState` 훅을 사용해줘"
        score = calculate_context_score(prompt)
        assert score >= 2.0


class TestEfficiencyScore:
    """Tests for calculate_efficiency_score function."""

    def test_short_conversation(self):
        """Should give high score for short conversations."""
        assert calculate_efficiency_score(2) == 10.0
        assert calculate_efficiency_score(1) == 10.0

    def test_medium_conversation(self):
        """Should give medium score for medium conversations."""
        assert calculate_efficiency_score(6) == 8.0
        assert calculate_efficiency_score(10) == 7.0
        assert calculate_efficiency_score(15) == 6.0

    def test_long_conversation(self):
        """Should give low score for long conversations."""
        assert calculate_efficiency_score(60) == 3.0
        assert calculate_efficiency_score(100) == 2.0
        assert calculate_efficiency_score(150) == 1.0


class TestDiversityScore:
    """Tests for calculate_diversity_score function."""

    def test_high_diversity(self):
        """Should give high score for many languages."""
        assert calculate_diversity_score(4) == 10.0
        assert calculate_diversity_score(5) == 10.0

    def test_medium_diversity(self):
        """Should give medium score for some languages."""
        assert calculate_diversity_score(3) == 8.0
        assert calculate_diversity_score(2) == 6.0

    def test_low_diversity(self):
        """Should give low score for single language."""
        assert calculate_diversity_score(1) == 4.0
        assert calculate_diversity_score(0) == 0.0


class TestProductivityScore:
    """Tests for calculate_productivity_score function."""

    def test_high_productivity(self):
        """Should give high score for many lines."""
        assert calculate_productivity_score(1000, 1000) == 10.0
        assert calculate_productivity_score(500, 1000) == 5.0

    def test_low_productivity(self):
        """Should give low score for few lines."""
        assert calculate_productivity_score(100, 1000) == 1.0
        assert calculate_productivity_score(0, 1000) == 0.0

    def test_cap_at_max(self):
        """Should cap score at 10."""
        assert calculate_productivity_score(2000, 1000) == 10.0


class TestCompositeScoreV2:
    """Tests for calculate_composite_score_v2 function."""

    def test_returns_all_scores(self):
        """Should return all individual scores and composite."""
        result = calculate_composite_score_v2(
            prompt="버그를 수정해줘",
            code_count=5,
            total_lines=100,
            message_count=10,
            language_diversity=2,
            max_lines=1000,
        )

        assert 'structure_score' in result
        assert 'context_score' in result
        assert 'productivity_score' in result
        assert 'efficiency_score' in result
        assert 'diversity_score' in result
        assert 'composite_score' in result

    def test_high_quality_prompt_scores_higher(self):
        """High quality prompts should score higher than low quality."""
        high_quality = calculate_composite_score_v2(
            prompt="LoginForm.tsx 컴포넌트에 비밀번호 유효성 검사를 추가해줘. React를 사용하고 있어.",
            code_count=5,
            total_lines=150,
            message_count=8,
            language_diversity=2,
            max_lines=1000,
        )

        low_quality = calculate_composite_score_v2(
            prompt="fix it",
            code_count=1,
            total_lines=10,
            message_count=50,
            language_diversity=1,
            max_lines=1000,
        )

        assert high_quality['composite_score'] > low_quality['composite_score']


class TestScoringAccuracy:
    """Integration tests for scoring accuracy with fixtures."""

    def test_high_quality_prompts_score_higher(self, high_quality_prompts, low_quality_prompts):
        """High quality prompts should score higher on average."""
        high_scores = []
        for p in high_quality_prompts:
            result = calculate_composite_score_v2(
                prompt=p['prompt'],
                code_count=p['code_count'],
                total_lines=p['total_lines'],
                message_count=p['message_count'],
                language_diversity=p['language_diversity'],
            )
            high_scores.append(result['composite_score'])

        low_scores = []
        for p in low_quality_prompts:
            result = calculate_composite_score_v2(
                prompt=p['prompt'],
                code_count=p['code_count'],
                total_lines=p['total_lines'],
                message_count=p['message_count'],
                language_diversity=p['language_diversity'],
            )
            low_scores.append(result['composite_score'])

        high_avg = sum(high_scores) / len(high_scores)
        low_avg = sum(low_scores) / len(low_scores)

        assert high_avg > low_avg, f"High quality avg ({high_avg:.2f}) should be > low quality avg ({low_avg:.2f})"

    def test_structure_score_differentiation(self, high_quality_prompts, low_quality_prompts):
        """Structure score should differentiate prompt quality."""
        high_structure_scores = [
            calculate_structure_score(p['prompt']) for p in high_quality_prompts
        ]
        low_structure_scores = [
            calculate_structure_score(p['prompt']) for p in low_quality_prompts
        ]

        high_avg = sum(high_structure_scores) / len(high_structure_scores)
        low_avg = sum(low_structure_scores) / len(low_structure_scores)

        assert high_avg > low_avg, f"High quality structure avg ({high_avg:.2f}) should be > low quality ({low_avg:.2f})"


class TestQualityPenalty:
    """Tests for quality penalty in composite score."""

    def test_penalty_applied_for_low_structure(self):
        """Should apply penalty when structure < 2.0."""
        result = calculate_composite_score_v2(
            prompt="fix it",  # Very short, low structure
            code_count=5,
            total_lines=100,
            message_count=3,
            language_diversity=2,
        )
        assert result.get('quality_penalty', 1.0) < 1.0

    def test_penalty_applied_for_low_context(self):
        """Should apply penalty when context < 1.0."""
        result = calculate_composite_score_v2(
            prompt="그거 해줘",  # Context dependent, low context
            code_count=5,
            total_lines=100,
            message_count=3,
            language_diversity=2,
        )
        assert result.get('quality_penalty', 1.0) < 1.0

    def test_no_penalty_for_high_quality(self):
        """Should not apply penalty for high quality prompts."""
        result = calculate_composite_score_v2(
            prompt="LoginForm.tsx 컴포넌트에 비밀번호 유효성 검사를 추가해줘. React를 사용하고 있어.",
            code_count=5,
            total_lines=100,
            message_count=3,
            language_diversity=2,
        )
        assert result.get('quality_penalty', 1.0) == 1.0

    def test_heavy_penalty_for_very_low_quality(self):
        """Should apply heavy penalty when both structure and context < 1.0."""
        result = calculate_composite_score_v2(
            prompt="ok",  # Very short, both scores very low
            code_count=5,
            total_lines=100,
            message_count=3,
            language_diversity=2,
        )
        assert result.get('quality_penalty', 1.0) == 0.6

    def test_penalty_reduces_composite_score(self):
        """Penalty should reduce composite score."""
        low_quality = calculate_composite_score_v2(
            prompt="fix",
            code_count=10,
            total_lines=200,
            message_count=2,
            language_diversity=3,
        )
        high_quality = calculate_composite_score_v2(
            prompt="LoginForm.tsx에 validation을 추가해줘. React 프로젝트야.",
            code_count=10,
            total_lines=200,
            message_count=2,
            language_diversity=3,
        )
        # Even with same productivity/efficiency/diversity, high quality should score higher
        assert high_quality['composite_score'] > low_quality['composite_score']



"""Tests for prompt pattern extraction."""

import pytest
from claude_x.patterns import (
    extract_pattern_type,
    extract_template,
    extract_tags,
    calculate_pattern_quality,
    analyze_prompt_for_pattern,
    extract_patterns_from_prompts,
    PATTERN_TYPES,
)
from claude_x.classifier import PromptCategory


class TestExtractPatternType:
    """Tests for extract_pattern_type function."""

    def test_target_action_pattern_korean(self):
        """Should detect target + action patterns in Korean."""
        prompts = [
            "LoginForm.tsx에 validation 추가해줘",
            "components/Button.jsx를 수정해줘",
            "src/utils/helper.py에서 함수 찾아줘",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'target_action', f"'{prompt}' should be target_action, got {result}"

    def test_reference_based_pattern(self):
        """Should detect reference-based patterns."""
        prompts = [
            "기존 LoginForm처럼 SignupForm을 만들어줘",
            "create a component like the existing Header",
            "참고해서 비슷하게 만들어줘",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'reference_based', f"'{prompt}' should be reference_based, got {result}"

    def test_constraint_based_pattern(self):
        """Should detect constraint-based patterns."""
        prompts = [
            "외부 라이브러리 없이 구현해줘",
            "implement this without using any external dependencies",
            "최소 8자 이상으로 만들어줘",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'constraint_based', f"'{prompt}' should be constraint_based, got {result}"

    def test_question_driven_pattern(self):
        """Should detect question-driven patterns."""
        prompts = [
            "이게 뭐야?",
            "what is the difference?",
            "왜 이렇게 동작하는 거야?",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'question_driven', f"'{prompt}' should be question_driven, got {result}"

    def test_debug_report_pattern(self):
        """Should detect debug report patterns."""
        prompts = [
            "에러가 발생해요. TypeError: Cannot read property",
            "기대동작과 실제동작이 달라요",
            "stack trace를 분석해줘",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'debug_report', f"'{prompt}' should be debug_report, got {result}"

    def test_context_goal_pattern(self):
        """Should detect context + goal patterns."""
        prompts = [
            "현재 상황이 이런데 해결책을 찾아줘",
            "지금 이런 문제가 있는데 수정해줘",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'context_goal', f"'{prompt}' should be context_goal, got {result}"

    def test_generic_pattern(self):
        """Should return generic for unclassifiable prompts."""
        prompts = [
            "ok",
            "됐어",
            "hello",
        ]
        for prompt in prompts:
            result = extract_pattern_type(prompt)
            assert result == 'generic', f"'{prompt}' should be generic, got {result}"

    def test_empty_prompt(self):
        """Should return generic for empty prompts."""
        assert extract_pattern_type("") == 'generic'
        assert extract_pattern_type(None) == 'generic'


class TestExtractTemplate:
    """Tests for extract_template function."""

    def test_replaces_file_names(self):
        """Should replace file names with placeholders."""
        prompt = "helper.tsx에 validation 추가해줘"
        template = extract_template(prompt, 'target_action')
        assert '[FILE_NAME]' in template
        assert 'helper.tsx' not in template

    def test_replaces_paths(self):
        """Should replace paths with placeholders."""
        prompt = "src/components/Button.tsx를 수정해줘"
        template = extract_template(prompt, 'target_action')
        assert '[PATH]' in template

    def test_replaces_component_names(self):
        """Should replace component names with placeholders."""
        prompt = "UserProfileComponent를 만들어줘"
        template = extract_template(prompt, 'target_action')
        assert '[COMPONENT]' in template

    def test_replaces_urls(self):
        """Should replace URLs with placeholders."""
        prompt = "https://example.com/api 를 호출해줘"
        template = extract_template(prompt, 'generic')
        assert '[URL]' in template

    def test_replaces_code_blocks(self):
        """Should replace code blocks with placeholders."""
        prompt = "```typescript\nconst x = 1;\n``` 이 코드를 수정해줘"
        template = extract_template(prompt, 'generic')
        assert '[CODE_BLOCK]' in template

    def test_replaces_inline_code(self):
        """Should replace inline code with placeholders."""
        prompt = "`useState` 훅을 사용해줘"
        template = extract_template(prompt, 'generic')
        assert '[CODE]' in template

    def test_empty_prompt(self):
        """Should return empty string for empty prompts."""
        assert extract_template("", 'generic') == ""
        assert extract_template(None, 'generic') == ""


class TestExtractTags:
    """Tests for extract_tags function."""

    def test_extracts_tech_tags(self):
        """Should extract technology tags."""
        prompt = "React 컴포넌트에서 TypeScript 타입 에러"
        tags = extract_tags(prompt)
        assert 'react' in tags
        assert 'typescript' in tags

    def test_extracts_action_tags(self):
        """Should extract action tags."""
        prompts_tags = [
            ("새로 만들어줘", 'create'),
            ("버그 수정해줘", 'fix'),
            ("리팩토링해줘", 'refactor'),
            ("설명해줘", 'explain'),
        ]
        for prompt, expected_tag in prompts_tags:
            tags = extract_tags(prompt)
            assert expected_tag in tags, f"'{prompt}' should have tag '{expected_tag}', got {tags}"

    def test_extracts_multiple_tags(self):
        """Should extract multiple relevant tags."""
        prompt = "React 컴포넌트의 버그를 수정하고 테스트 추가해줘"
        tags = extract_tags(prompt)
        assert 'react' in tags
        assert 'fix' in tags
        assert 'test' in tags

    def test_empty_prompt(self):
        """Should return empty set for empty prompts."""
        assert extract_tags("") == set()
        assert extract_tags(None) == set()


class TestCalculatePatternQuality:
    """Tests for calculate_pattern_quality function."""

    def test_high_quality_prompt(self):
        """Should give high score to well-structured prompts."""
        prompt = "LoginForm.tsx 컴포넌트에 비밀번호 유효성 검사를 추가해줘. React를 사용중이야."
        score = calculate_pattern_quality(prompt)
        assert score >= 5.0, f"Expected >= 5.0, got {score}"

    def test_low_quality_prompt(self):
        """Should give low score to vague prompts."""
        prompt = "고쳐줘"
        score = calculate_pattern_quality(prompt)
        assert score <= 3.0, f"Expected <= 3.0, got {score}"

    def test_empty_prompt(self):
        """Should return 0 for empty prompts."""
        assert calculate_pattern_quality("") == 0.0
        assert calculate_pattern_quality(None) == 0.0


class TestAnalyzePromptForPattern:
    """Tests for analyze_prompt_for_pattern function."""

    def test_returns_all_fields(self):
        """Should return all expected fields."""
        prompt = "LoginForm.tsx에 validation 추가해줘"
        result = analyze_prompt_for_pattern(prompt)

        assert 'pattern_type' in result
        assert 'pattern_description' in result
        assert 'template' in result
        assert 'category' in result
        assert 'category_icon' in result
        assert 'tags' in result
        assert 'quality_score' in result

    def test_correct_pattern_type(self):
        """Should identify correct pattern type."""
        prompt = "기존 Button처럼 새 컴포넌트를 만들어줘"
        result = analyze_prompt_for_pattern(prompt)
        assert result['pattern_type'] == 'reference_based'

    def test_correct_category(self):
        """Should identify correct category."""
        prompt = "버그를 수정해줘"
        result = analyze_prompt_for_pattern(prompt)
        assert result['category'] == PromptCategory.DEBUGGING.value

    def test_empty_prompt(self):
        """Should handle empty prompts gracefully."""
        result = analyze_prompt_for_pattern("")
        assert result['pattern_type'] == 'generic'
        assert result['quality_score'] == 0.0


class TestExtractPatternsFromPrompts:
    """Tests for extract_patterns_from_prompts function."""

    def test_extracts_patterns(self, high_quality_prompts):
        """Should extract patterns from high quality prompts."""
        prompts = [{'first_prompt': p['prompt']} for p in high_quality_prompts]
        patterns = extract_patterns_from_prompts(prompts, min_quality=3.0)

        assert len(patterns) > 0

    def test_filters_by_quality(self, low_quality_prompts):
        """Should filter out low quality prompts."""
        prompts = [{'first_prompt': p['prompt']} for p in low_quality_prompts]
        patterns = extract_patterns_from_prompts(prompts, min_quality=8.0)

        # Low quality prompts should be filtered out
        assert len(patterns) < len(prompts)

    def test_groups_similar_patterns(self):
        """Should group similar patterns together."""
        prompts = [
            {'first_prompt': "LoginForm.tsx에 validation 추가해줘"},
            {'first_prompt': "SignupForm.tsx에 validation 추가해줘"},
            {'first_prompt': "ProfileForm.tsx에 validation 추가해줘"},
        ]
        patterns = extract_patterns_from_prompts(prompts, min_quality=0.0)

        # Similar patterns should be grouped
        # May have 1-3 patterns depending on template similarity
        assert len(patterns) <= len(prompts)

    def test_empty_list(self):
        """Should handle empty list gracefully."""
        patterns = extract_patterns_from_prompts([])
        assert patterns == []
